@@ -16,7 +16,8 @@
 """
 WMS service handler
 """
-from itertools import chain
+from mapproxy.compat import iteritems
+from mapproxy.compat.itertools import chain
 from functools import partial
 from mapproxy.cache.tile import CacheInfo
 from mapproxy.request.wms import (wms_request, WMS111LegendGraphicRequest,
@@ -147,7 +148,7 @@ class WMSServer(Server):
 
         try:
             result_buf = result.as_buffer(img_opts)
-        except IOError, ex:
+        except IOError as ex:
             raise RequestError('error while processing image file: %s' % ex,
                 request=map_request)
 
@@ -169,7 +170,7 @@ class WMSServer(Server):
         # if '__debug__' in map_request.params:
         #     layers = self.layers.values()
         # else:
-        #     layers = [layer for name, layer in self.layers.iteritems()
+        #     layers = [layer for name, layer in iteritems(self.layers)
         #               if name != '__debug__']
 
         if map_request.params.get('tiled', 'false').lower() == 'true':
@@ -329,7 +330,7 @@ class WMSServer(Server):
                 return PERMIT_ALL_LAYERS, None
             layers = {}
             if result['authorized'] == 'partial':
-                for layer_name, permissions in result['layers'].iteritems():
+                for layer_name, permissions in iteritems(result['layers']):
                     if permissions.get(feature, False) == True:
                         layers[layer_name] = permissions.get('limited_to')
             limited_to = result.get('limited_to')
@@ -451,7 +452,7 @@ def limit_srs_extents(srs_extents, supported_srs):
     else:
         srs_extents = DEFAULT_EXTENTS.copy()
 
-    for srs in srs_extents.keys():
+    for srs in list(srs_extents.keys()):
         if srs not in supported_srs:
             srs_extents.pop(srs)
 
@@ -473,7 +474,7 @@ class Capabilities(object):
 
     def layer_srs_bbox(self, layer, epsg_axis_order=False):
         layer_srs_code = layer.extent.srs.srs_code
-        for srs, extent in self.srs_extents.iteritems():
+        for srs, extent in iteritems(self.srs_extents):
             if extent.is_default:
                 bbox = layer.extent.bbox_for(SRS(srs))
             else:
@@ -540,8 +541,8 @@ class LayerRenderer(object):
                 else:
                     ex = layer_task.exception
                     async_pool.shutdown(True)
-                    raise ex[0], ex[1], ex[2]
-        except SourceError, ex:
+                    raise ex[1]
+        except SourceError as ex:
             raise RequestError(ex.args[0], request=self.request)
 
     def _render_capture_source_errors(self, async_pool, render_layers, layer_merger):
@@ -563,7 +564,7 @@ class LayerRenderer(object):
                     errors.append(ex[1].args[0])
                 else:
                     async_pool.shutdown(True)
-                    raise ex[0], ex[1], ex[2]
+                    raise ex[1]
 
         if render_layers and not rendered:
             errors = '\n'.join(errors)
@@ -584,7 +585,7 @@ class LayerRenderer(object):
             raise
         except MapBBOXError:
             raise RequestError('Request too large or invalid BBOX.', request=self.request)
-        except MapError, e:
+        except MapError as e:
             raise RequestError('Invalid request: %s' % e.args[0], request=self.request)
         except TransformationError:
             raise RequestError('Could not transform BBOX: Invalid result.',
